package cubo3d;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import javax.swing.JFrame;
import javax.swing.JPanel;

public class Principal extends JPanel implements MouseListener, MouseMotionListener, KeyListener {
    
    private final double D = 1000;
    
    private Forma3D forma;
    private int mouseX = -1;
    private int mouseY = -1;
    private final Transformador transformador = new Transformador();
    
    private char rotCH = 'w';
    
    public Principal() {
        super.setBackground( Color.BLACK );
        super.addMouseListener( this ); 
        super.addMouseMotionListener( this ); 
        super.addKeyListener( this );
        super.setFocusable( true ); 
    }
    
    @Override
    public void paintComponent( Graphics g ) {
        super.paintComponent( g );
        
        if ( forma != null ) {
            int xc = super.getWidth()  / 2;
            int yc = super.getHeight() / 2;
            
            forma.getVertices().forEach( (v) -> { v.carregaCopia(); } ); 
            transformador.perspectiva( forma, D );
            
            for( Aresta3D a : forma.getArestas() ) {
                double[] v1 = a.getV1().getVetorCopia();
                double[] v2 = a.getV2().getVetorCopia();
                               
                int x1 = xc + (int)v1[0];
                int y1 = yc - (int)v1[1];
                int x2 = xc + (int)v2[0];
                int y2 = yc - (int)v2[1];
                
                g.setColor( Color.WHITE );
                g.drawLine( x1, y1, x2, y2 );
            }
        }
    }

    @Override
    public void mouseDragged(MouseEvent e) {
        if ( mouseX != -1 && forma != null ) {
            int w = super.getWidth();
            int h = super.getHeight();
            int dx = e.getX() - mouseX;
            int dy = e.getY() - mouseY;
            
            double yang = ( (double)dx / (double)w ) * 2 * Math.PI;
            double xang = ( (double)dy / (double)h ) * 2 * Math.PI;
            
            //transformador.rot( forma, xang, yang, 0 );
            
            char r = Character.toLowerCase( rotCH );
            switch ( r ) {
                case 'z':
                    transformador.rotZ( forma, -yang ); 
                    break;
                default:
                    transformador.rotX( forma, -xang );
                    transformador.rotY( forma, -yang );                     
            }            
            
            super.repaint();
        }        
        mouseX = e.getX();
        mouseY = e.getY();        
    }
    
    @Override
    public void mouseReleased(MouseEvent e) {
        mouseX = -1;
        mouseY = -1;
    }
    
    @Override
    public void keyPressed(KeyEvent e ) {
        rotCH = e.getKeyChar();
    }

    @Override
    public void keyTyped(KeyEvent e) {}

    @Override
    public void keyReleased(KeyEvent e) {}    

    @Override
    public void mouseMoved(MouseEvent e) {}

    @Override
    public void mouseClicked(MouseEvent e) {}

    @Override
    public void mousePressed(MouseEvent e) {}
    
    @Override
    public void mouseEntered(MouseEvent e) {}

    @Override
    public void mouseExited(MouseEvent e) {}
    
    public void setForma3D( Forma3D forma ) {
        this.forma = forma;
    }
    
    public static void main(String[] args) {
        Cubo3D cubo = new Cubo3D();
        cubo.constroi( 200 ); 
        
        Principal p = new Principal();
        p.setForma3D( cubo );
        
        JFrame f = new JFrame();
        f.setTitle( "Desenho de cubo 3D" ); 
        f.setContentPane( p );
        f.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
        f.setSize( 640, 480 );
        f.setLocationRelativeTo( f );
        f.setVisible( true );
        
        p.repaint();
    }
    
}
